#===============================================================================
# Ttulo   : Anlise quantitativa de dados - AQE - Ajustamento
# Autor    : Jos Cludio Faria/UESC/DCET
# Data     : 2023-07-10 17:10:56
# Objetivos:
#===============================================================================
# a) Apresentar os recursos bsicos do R para anlise de regresso
# b) Fundamentao em regresso linear simples
# c) Fundamentao em lgebra de matrizes
#===============================================================================

#===============================================================================
#.. nfase em lgebra de matrizes
#.. +----------------------+
#.. |             -1       |
#.. | b_est = (X'X) . X'Y  |  BLUE - Best Linear Unbiased Estimators
#.. +----------------------+
# b_est: soluo matricial do mtodo dos mnimos quadrados dos erros.
# Operaes matriciais permitem a obteno das estimativas (b_est) dos parmetros
# dos modelos lineares.
#===============================================================================

x <- c(10, 20, 30, 40, 50, 60, 70)
y <- c(1000, 2300, 3600, 4500, 5400, 5800, 5600)

(X <- cbind(1, x))        # Modelo linear de 1 grau: com intercepto
# (X <- cbind(x))           # Modelo linear de 1 grau: forando para a origem
# (X <- cbind(1, x, x^2))   # Modelo linear de 2 grau: com intercepto
# (X <- cbind(x, x^2))      # Modelo linear de 2 grau: forando para a origem

(Y <- cbind(y))

(XlX     <- t(X) %*% X)      # X'X

                             #     -1
(XlX_inv <- solve(XlX))      # (X'X) 

(XlY     <- t(X) %*% Y)      #  X'Y

                             #      -1     
(b_est   <- XlX_inv %*% XlY) # (X'X) . X'Y

(Y_est   <- X %*% b_est)     # Ys estimados pelo modelo ajustado

(res     <- Y - Y_est)       # Erros

# Visualizando os dados
plot(y ~ x,
     xlim=c(0, 70),
     ylim=c(0, 7000),
     pch=19,
     col='darkgreen')

# Sobrepondo os valores estimados pelo modelo ajustado (curva suavizada)
lines(spline(Y_est ~ x),
      col='red')

points(Y_est ~ x,
       col='red',
       pch=19)


#===============================================================================
#.. nfase na funo de alto nvel do R - lm (package:stats)
#===============================================================================
rl <- lm(y ~ x)               # Modelo linear de 1 grau: com intercepto
# rl <- lm(y ~ 0 + x)           # Modelo linear de 1 grau: forando para a origem 
# rl <- lm(y ~ x + I(x^2))      # Modelo linear de 2 grau: com intercepto                        
# rl <- lm(y ~ 0 + x + I(x^2))  # Modelo linear de 2 grau: forando para a origem                       

summary(rl)
class(rl)
mode(rl)
names(rl)

# Visualizando os dados
plot(y ~ x,
     xlim=c(0, 70),
     ylim=c(0, 7000),
     pch=19,
     col='darkgreen')

# Sobrepondo o modelo ajustado
lines(spline(fitted(rl) ~ x,
             n=1e2),
      col='red')

points(fitted(rl) ~ x,
       col='red',
       pch=19)

#===============================================================================
# Verificando apresentao grfica com todos os modelos
#===============================================================================
plot(y ~ x,
     xlim=c(0, 70),
     ylim=c(0, 7000),
     pch=19,
     col='darkgreen')

# Modelo linear de 1 grau: com intercepto
lines(fitted(lm(y ~ x)) ~ x,
      col='blue')

# Modelo linear de 1 grau: ajustado para a origem
lines(fitted(lm(y ~ 0 + x)) ~ x,
      col='blue',
      lty=2)

# Modelo linear de 2 grau: com intercepto
lines(spline(fitted(lm(y ~ x + I(x^2))) ~ x,
             n=1e2),
      col='red',
      lw=2)

# Modelo linear de 2 grau: ajustado para a origem
lines(spline(fitted(lm(y ~ 0 + x + I(x^2))) ~ x, 
             n=1e2),
      col='red',
      lw=2,
      lty=2)

